@@ -51,6 +51,11 @@
 
 typedef struct {
 	GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon;
+	gboolean display_internal_hard_drives;
+	gboolean display_scsi_drives;
+	gboolean display_scsi_optical_drives;
+	gboolean display_external_drives;
+	gboolean display_drives_removable;
 	guint gconf_client_connection_id;
 } GnomeVFSHalUserData;
 
@@ -220,6 +225,10 @@ _hal_drive_policy_get_icon (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon,
 	LibHalDriveBus bus;
 	LibHalDriveType drive_type;
 
+	name = libhal_drive_get_dedicated_icon_drive (hal_drive);
+	if (name != NULL)
+		goto out;
+
 	bus        = libhal_drive_get_bus (hal_drive);
 	drive_type = libhal_drive_get_type (hal_drive);
 
@@ -249,6 +258,7 @@ _hal_drive_policy_get_icon (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon,
 		name = _hal_lookup_icon (0x10000 + drive_type*0x100);
 	}
 
+out:
 	if (name != NULL)
 		return g_strdup (name);
 	else {
@@ -266,6 +276,10 @@ _hal_volume_policy_get_icon (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon,
 	LibHalDriveType drive_type;
 	LibHalVolumeDiscType disc_type;
 
+	name = libhal_drive_get_dedicated_icon_volume (hal_drive);
+	if (name != NULL)
+		goto out;
+
 	/* by design, the enums are laid out so we can do easy computations */
 
 	if (libhal_volume_is_disc (hal_volume)) {
@@ -300,39 +314,6 @@ out:
 	}
 }
 
-/*------------------------------------------------------------------------*/
-static char *
-_hal_size_as_string (gint64 size)
-{
-	char *result;
-	char* sizes_str[] = {"K", "M", "G", "T", NULL};
-	gint64 cur = 1000L;
-	gint64 base = 10L;
-	gint64 step = 10L*10L*10L;
-	int cur_str = 0;
-
-	result = NULL;
-
-	do {
-		if (sizes_str[cur_str+1] == NULL || size < cur*step) {
-			/* found the unit, display a comma number if result is a single digit */
-			if (size < cur*base) {
-				result = g_strdup_printf ("%.01f%s", ((double)size)/((double)cur), sizes_str[cur_str]);
-			} else {
-				result = g_strdup_printf ("%lld%s", size / cur, sizes_str[cur_str]);
-			}
-			goto out;
-		}
-
-		cur *= step;
-		cur_str++;
-
-	} while (TRUE);
-
-out:
-	return result;
-}
-
 /*------------------------------------------------------------------------*/
 /* hal_volume may be NULL */
 static char *
@@ -368,7 +349,7 @@ _hal_drive_policy_get_display_name (GnomeVFSVolumeMonitorDaemon *volume_monitor_
 
 		/* Otherwise, just use volume size */
 
-		size_str = _hal_size_as_string (libhal_volume_get_size (hal_volume));
+		size_str = gnome_vfs_format_file_size_for_display (libhal_volume_get_size (hal_volume));
 		if (size_str != NULL) {
 			name = g_strdup_printf (_("%s Volume"), size_str);
 		}
@@ -381,7 +362,6 @@ _hal_drive_policy_get_display_name (GnomeVFSVolumeMonitorDaemon *volume_monitor_
 	/* drives we know the type of */
 	if (drive_type == LIBHAL_DRIVE_TYPE_CDROM) {
 		const char *first;
-		const char *sep;
 		const char *second;
 		LibHalDriveCdromCaps drive_cdrom_caps;
 			
@@ -393,8 +373,6 @@ _hal_drive_policy_get_display_name (GnomeVFSVolumeMonitorDaemon *volume_monitor_
 		if (drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_CDRW)
 			first = _("CD-RW");
 		
-		sep = _("/");
-			
 		second = NULL;
 		if (drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_DVDROM)
 			second = _("DVD-ROM");
@@ -415,7 +393,12 @@ _hal_drive_policy_get_display_name (GnomeVFSVolumeMonitorDaemon *volume_monitor_
 		    (drive_cdrom_caps & LIBHAL_DRIVE_CDROM_CAPS_DVDPLUSRW))
 			second = _("DVD±RW");
 
-		name = g_strdup_printf (_("%s%s%s Drive"), first, second != NULL ? sep : "", second);
+		if (second != NULL) {
+			name = g_strdup_printf (_("%s/%s Drive"), first, second);
+		} else {
+			name = g_strdup_printf (_("%s Drive"), first);
+		}
+			
 		may_prepend_external = TRUE;		
 	} else if (drive_type == LIBHAL_DRIVE_TYPE_FLOPPY) {
 		name = g_strdup (_("Floppy Drive"));
@@ -581,7 +564,7 @@ _hal_volume_policy_get_display_name (GnomeVFSVolumeMonitorDaemon *volume_monitor
 
 	/* Fallback: size of media */
 
-	size_str = _hal_size_as_string (libhal_volume_get_size (hal_volume));
+	size_str = gnome_vfs_format_file_size_for_display (libhal_volume_get_size (hal_volume));
 	if (size_str != NULL) {
 		if (libhal_drive_uses_removable_media (hal_drive)) {
 			name = g_strdup_printf (_("%s Removable Volume"), size_str);
@@ -608,55 +591,35 @@ _hal_drive_policy_check (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon,
 			 LibHalDrive *hal_drive, LibHalVolume *hal_volume)
 {
 	gboolean ret;
-	gboolean display_internal_hard_drives;
-	gboolean display_scsi_drives;
-	gboolean display_scsi_optical_drives;
-	gboolean display_external_drives;
-	gboolean display_drives_removable;
+	GnomeVFSHalUserData *hal_userdata;
 
 	ret = FALSE;
 
-	/* respect the gconf settings! */
-	display_internal_hard_drives = gconf_client_get_bool (volume_monitor_daemon->gconf_client,
-							      PATH_GCONF_GNOME_VFS_STORAGE 
-							      "/display_internal_hard_drives",
-							      NULL);
-	display_scsi_drives = gconf_client_get_bool (volume_monitor_daemon->gconf_client,
-						     PATH_GCONF_GNOME_VFS_STORAGE 
-						     "/display_scsi_drives",
-						     NULL);
-	display_scsi_optical_drives = gconf_client_get_bool (volume_monitor_daemon->gconf_client,
-							     PATH_GCONF_GNOME_VFS_STORAGE 
-							     "/display_scsi_optical_drives",
-							     NULL);
-	display_external_drives = gconf_client_get_bool (volume_monitor_daemon->gconf_client,
-							 PATH_GCONF_GNOME_VFS_STORAGE
-							 "/display_external_drives",
-							 NULL);
-	display_drives_removable = gconf_client_get_bool (volume_monitor_daemon->gconf_client,
-							  PATH_GCONF_GNOME_VFS_STORAGE
-							  "/display_drives_with_removable_media",
-							  NULL);
+	g_return_val_if_fail (volume_monitor_daemon != NULL, ret);
 
-	if (!display_external_drives) {
+	hal_userdata = (GnomeVFSHalUserData *) libhal_ctx_get_user_data (volume_monitor_daemon->hal_ctx);
+
+	g_return_val_if_fail (hal_userdata != NULL, ret);
+
+	if (!hal_userdata->display_external_drives) {
 		if (libhal_drive_is_hotpluggable (hal_drive))
 			goto out;
 	}
 
-	if (!display_drives_removable) {
+	if (!hal_userdata->display_drives_removable) {
 		if (libhal_drive_uses_removable_media (hal_drive))
 			goto out;
 	}
 
-	if (!display_internal_hard_drives) {
+	if (!hal_userdata->display_internal_hard_drives) {
 		if ((libhal_drive_uses_removable_media (hal_drive) == FALSE) && 
 		    (libhal_drive_is_hotpluggable (hal_drive) == FALSE))
 			goto out;
 	}
 
-	if (!display_scsi_drives) {
+	if (!hal_userdata->display_scsi_drives) {
 		if (libhal_drive_get_bus (hal_drive) == LIBHAL_DRIVE_BUS_SCSI) {
-			if (display_scsi_optical_drives) {
+			if (hal_userdata->display_scsi_optical_drives) {
 				if (libhal_drive_get_type (hal_drive) == LIBHAL_DRIVE_TYPE_CDROM) {
 					/* we're safe */
 				} else {
@@ -830,8 +793,8 @@ _hal_add_drive_without_volumes (GnomeVFSVolumeMonitorDaemon *volume_monitor_daem
 
 	g_return_if_fail (hal_drive != NULL);
 
-	g_debug ("entering _hal_add_drive_without_volumes for\n  drive udi '%s'\n",
-		 libhal_drive_get_udi (hal_drive));
+	/* g_debug ("entering _hal_add_drive_without_volumes for\n  drive udi '%s'\n", 
+	   libhal_drive_get_udi (hal_drive)); */
 
 	volume_monitor = GNOME_VFS_VOLUME_MONITOR (volume_monitor_daemon);
 	hal_userdata = (GnomeVFSHalUserData *) libhal_ctx_get_user_data (volume_monitor_daemon->hal_ctx);
@@ -856,7 +819,7 @@ _hal_add_drive_without_volumes (GnomeVFSVolumeMonitorDaemon *volume_monitor_daem
 		goto out;
 	
 	drive = g_object_new (GNOME_VFS_TYPE_DRIVE, NULL);
-	drive->priv->activation_uri = gnome_vfs_get_uri_from_local_path (HAL_MOUNTS_INVALID_URI); /* (!) */
+	drive->priv->activation_uri = g_strdup ("");
 	drive->priv->is_connected = 1;
 	drive->priv->device_path = g_strdup (libhal_drive_get_device_file (hal_drive));
 	drive->priv->device_type = _hal_get_gnome_vfs_device_type (hal_drive);
@@ -909,8 +872,8 @@ _hal_add_volume (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon,
 		goto out;
 	}
 
-	//g_debug ("entering _hal_add_volume for\n  drive udi '%s'\n  volume udi '%s'\n",
-	//	 libhal_drive_get_udi (drive), libhal_volume_get_udi (volume));
+	/* g_debug ("entering _hal_add_volume for\n  drive udi '%s'\n  volume udi '%s'\n",
+	   libhal_drive_get_udi (drive), libhal_volume_get_udi (volume)); */
 
 	/* OK, check if we got a drive_without_volumes drive and delete that since we're going to add a
 	 * drive for added partitions */
@@ -919,27 +882,11 @@ _hal_add_volume (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon,
 		_gnome_vfs_volume_monitor_disconnected (volume_monitor, drive);
 	}
 
-	/* if we had a drive from here but where we weren't mounted, just use that drive and update
-	 * activation_uri as we now have a definite mount point...
+	/* if we had a drive from here but where we weren't mounted, just use that drive since nothing actually
+	 * changed 
 	 */
 	drive = _gnome_vfs_volume_monitor_find_drive_by_hal_udi (volume_monitor, libhal_volume_get_udi (hal_volume));
-	if (drive != NULL) {
-		if (libhal_volume_disc_has_audio (hal_volume)) {
-			drive->priv->activation_uri = g_strdup_printf ("cdda://%s", 
-								       libhal_volume_get_device_file (hal_volume));
-		} else if (libhal_volume_disc_is_blank (hal_volume)) {
-			drive->priv->activation_uri = g_strdup ("burn:///");
-		} else if (libhal_volume_is_mounted (hal_volume)) {
-			drive->priv->activation_uri = gnome_vfs_get_uri_from_local_path (
-				libhal_volume_get_mount_point (hal_volume));
-		} else {
-			/* This sucks but it doesn't make sense to talk about the activation_uri if we're not mounted!
-			 * So fake it...
-			 */
-			drive->priv->activation_uri = gnome_vfs_get_uri_from_local_path (HAL_MOUNTS_INVALID_URI); 
-		}
-
-	} else {
+	if (drive == NULL) {
 		drive = g_object_new (GNOME_VFS_TYPE_DRIVE, NULL);
 		if (libhal_volume_disc_has_audio (hal_volume)) {
 			drive->priv->activation_uri = g_strdup_printf ("cdda://%s", 
@@ -951,14 +898,14 @@ _hal_add_volume (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon,
 				libhal_volume_get_mount_point (hal_volume));
 		} else {
 			/* This sucks but it doesn't make sense to talk about the activation_uri if we're not mounted!
-			 * So fake it...
+			 * So just set it to the empty string
 			 */
-			drive->priv->activation_uri = gnome_vfs_get_uri_from_local_path (HAL_MOUNTS_INVALID_URI); 
+			drive->priv->activation_uri = g_strdup ("");
 		}
 		drive->priv->is_connected = TRUE;
 		drive->priv->device_path = g_strdup (libhal_volume_get_device_file (hal_volume));
 		drive->priv->device_type = _hal_get_gnome_vfs_device_type (hal_drive);
-		
+	
 		/* TODO: could add an icon of a drive with media in it since this codepath only
 		 * handles drives with media in them
 		 */
@@ -970,12 +917,11 @@ _hal_add_volume (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon,
 		drive->priv->volumes = NULL;
 		drive->priv->hal_udi = g_strdup (libhal_volume_get_udi (hal_volume));
 		drive->priv->hal_drive_udi = g_strdup (libhal_drive_get_udi (hal_drive));
-		
+	
 		_gnome_vfs_volume_monitor_connected (volume_monitor, drive);
 		gnome_vfs_drive_unref (drive);
 	}
 
-
 	vol = _gnome_vfs_volume_monitor_find_volume_by_hal_udi (volume_monitor, libhal_volume_get_udi (hal_volume));
 	if (vol == NULL && 
 	    (libhal_volume_is_mounted (hal_volume) || 
@@ -1034,7 +980,7 @@ _hal_update_all (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon)
 	int num_drives;
 	DBusError error;
 
-	g_debug ("entering _hal_update_all");
+	/* g_debug ("entering _hal_update_all"); */
 
 	dbus_error_init (&error);
 	drives = libhal_find_device_by_capability (volume_monitor_daemon->hal_ctx,
@@ -1045,7 +991,7 @@ _hal_update_all (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon)
 		for (i = 0; i < num_drives; i++) {
 			LibHalDrive *drive;
 
-			g_debug ("drive = '%s'", drives[i]);
+			/* g_debug ("drive = '%s'", drives[i]); */
 			drive = libhal_drive_from_udi (volume_monitor_daemon->hal_ctx, drives[i]);
 			if (drive != NULL) {
 				char **volumes;
@@ -1062,7 +1008,7 @@ _hal_update_all (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon)
 					for (j = 0; j < num_volumes; j++) {
 						LibHalVolume *volume;
 
-						g_debug ("  volume = '%s'", volumes[j]);
+						/* g_debug ("  volume = '%s'", volumes[j]); */
 						volume = libhal_volume_from_udi (volume_monitor_daemon->hal_ctx, 
 										 volumes[j]);
 
@@ -1099,7 +1045,7 @@ _hal_device_added (LibHalContext *hal_ctx,
 	GnomeVFSHalUserData *hal_userdata;
 	GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon;
 
-	g_debug ("Entering %s", __FUNCTION__);
+	/* g_debug ("Entering %s", __FUNCTION__); */
 	
 	hal_userdata = (GnomeVFSHalUserData *) libhal_ctx_get_user_data (hal_ctx);
 	volume_monitor_daemon = hal_userdata->volume_monitor_daemon;
@@ -1184,7 +1130,7 @@ _hal_device_removed (LibHalContext *hal_ctx, const char *udi)
 	GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon;
 	char *hal_drive_udi;
 
-	g_debug ("Entering %s", __FUNCTION__);
+	/* g_debug ("Entering %s", __FUNCTION__); */
 	
 	hal_userdata = (GnomeVFSHalUserData *) libhal_ctx_get_user_data (hal_ctx);
 	volume_monitor_daemon = hal_userdata->volume_monitor_daemon;
@@ -1207,7 +1153,7 @@ _hal_device_removed (LibHalContext *hal_ctx, const char *udi)
 		_gnome_vfs_volume_monitor_disconnected (GNOME_VFS_VOLUME_MONITOR (volume_monitor_daemon), drive);
 	}
 
-	g_debug ("hal_drive_udi = %s", hal_drive_udi);
+	/* g_debug ("hal_drive_udi = %s", hal_drive_udi); */
 
 	/* if there are no other drives with the same hal_drive_udi as us, add a drive_without_volumes object  */
 	if (hal_drive_udi != NULL && 
@@ -1215,7 +1161,7 @@ _hal_device_removed (LibHalContext *hal_ctx, const char *udi)
 		    GNOME_VFS_VOLUME_MONITOR (volume_monitor_daemon), hal_drive_udi) == NULL) {
 		LibHalDrive *drive;
 
-		g_debug ("going to add drive_without_volumes for %s", hal_drive_udi);
+		/* g_debug ("going to add drive_without_volumes for %s", hal_drive_udi); */
 
 		drive = libhal_drive_from_udi (volume_monitor_daemon->hal_ctx, hal_drive_udi);
 		if (drive != NULL) {
@@ -1308,6 +1254,39 @@ out:
 }
 	
 
+static void 
+_hal_get_settings_from_gconf (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon)
+{
+	GnomeVFSHalUserData *hal_userdata;
+
+	g_return_if_fail (volume_monitor_daemon != NULL);
+
+	hal_userdata = (GnomeVFSHalUserData *) libhal_ctx_get_user_data (volume_monitor_daemon->hal_ctx);
+
+	g_return_if_fail (hal_userdata != NULL);
+
+	/* respect the gconf settings! */
+	hal_userdata->display_internal_hard_drives = gconf_client_get_bool (volume_monitor_daemon->gconf_client,
+									    PATH_GCONF_GNOME_VFS_STORAGE 
+									    "/display_internal_hard_drives",
+									    NULL);
+	hal_userdata->display_scsi_drives = gconf_client_get_bool (volume_monitor_daemon->gconf_client,
+								   PATH_GCONF_GNOME_VFS_STORAGE 
+								   "/display_scsi_drives",
+								   NULL);
+	hal_userdata->display_scsi_optical_drives = gconf_client_get_bool (volume_monitor_daemon->gconf_client,
+									   PATH_GCONF_GNOME_VFS_STORAGE 
+									   "/display_scsi_optical_drives",
+									   NULL);
+	hal_userdata->display_external_drives = gconf_client_get_bool (volume_monitor_daemon->gconf_client,
+								       PATH_GCONF_GNOME_VFS_STORAGE
+								       "/display_external_drives",
+								       NULL);
+	hal_userdata->display_drives_removable = gconf_client_get_bool (volume_monitor_daemon->gconf_client,
+									PATH_GCONF_GNOME_VFS_STORAGE
+									"/display_drives_with_removable_media",
+									NULL);
+}
 
 static void
 _hal_settings_changed (GConfClient* client,
@@ -1319,10 +1298,15 @@ _hal_settings_changed (GConfClient* client,
 
 	volume_monitor_daemon = data;
 
-	g_debug ("in _hal_settings_changed");
+	g_return_if_fail (volume_monitor_daemon != NULL);
+
+	/* g_debug ("in _hal_settings_changed"); */
+
+	_hal_get_settings_from_gconf (volume_monitor_daemon);
 	_gnome_vfs_hal_mounts_force_reprobe (volume_monitor_daemon);
 }
 
+
 gboolean
 _gnome_vfs_hal_mounts_init (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon)
 {
@@ -1330,7 +1314,7 @@ _gnome_vfs_hal_mounts_init (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon)
 	DBusConnection *dbus_connection;
 	GnomeVFSHalUserData *hal_userdata;
 
-	g_debug ("Entering %s", __FUNCTION__);
+	/* g_debug ("Entering %s", __FUNCTION__); */
 
 	/* Initialise the connection to the hal daemon */
 	if ((volume_monitor_daemon->hal_ctx = 
@@ -1384,13 +1368,14 @@ _gnome_vfs_hal_mounts_init (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon)
 					 NULL,
 					 NULL);
 
-
 	/* Simply watch all property changes instead of dynamically
 	 * adding/removing match rules bus-side to only match certain
 	 * objects...
 	 */
 	libhal_device_property_watch_all (volume_monitor_daemon->hal_ctx, &error);
 
+	_hal_get_settings_from_gconf (volume_monitor_daemon);
+
 	/* add drives/volumes from HAL */
 	_hal_update_all (volume_monitor_daemon);
 
@@ -1424,7 +1409,7 @@ _gnome_vfs_hal_mounts_shutdown (GnomeVFSVolumeMonitorDaemon *volume_monitor_daem
 void
 _gnome_vfs_hal_mounts_force_reprobe (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon)
 {
-	g_debug ("entering _gnome_vfs_hal_mounts_force_reprobe");
+	/* g_debug ("entering _gnome_vfs_hal_mounts_force_reprobe"); */
 	_hal_update_all (volume_monitor_daemon);
 }
 
