@@ -43,6 +43,7 @@
 #include <openssl/x509.h>
 #include <openssl/err.h>
 #elif defined HAVE_GNUTLS
+#include <gcrypt.h>
 #include <gnutls/gnutls.h>
 #endif
 #if defined(HAVE_OPENSSL) || defined(HAVE_GNUTLS)
@@ -73,6 +74,7 @@ typedef struct {
 	int sockfd;
 	gnutls_session tlsstate;
 	gnutls_certificate_client_credentials xcred;
+	struct timeval *timeout;
 #elif defined HAVE_NSS
 	PRFileDesc *sockfd;
 #else
@@ -84,12 +86,49 @@ struct GnomeVFSSSL {
 	GnomeVFSSSLPrivate *private;
 };
 
+#ifdef HAVE_GNUTLS
+static int gcry_gthread_mutex_init (void **priv)			      
+{									      
+	GMutex* lock = g_mutex_new();
+	if (!lock)								      
+		return ENOMEM;							      
+	*priv = lock;							      
+	return 0;								      
+}									      
+
+static int gcry_gthread_mutex_destroy (void **lock)			      
+{ 
+	g_mutex_free (*lock);
+	return 0; 
+}
+
+static int gcry_gthread_mutex_lock (void **lock)			      
+{ 
+	g_mutex_lock(*lock);
+	return 0;
+}				      
+
+static int gcry_gthread_mutex_unlock (void **lock)			      
+{ 
+	g_mutex_unlock(*lock);
+	return 0;
+}				      
+
+static struct gcry_thread_cbs gcry_threads_gthread =			      
+{
+	GCRY_THREAD_OPTION_USER, NULL,					      
+	gcry_gthread_mutex_init, gcry_gthread_mutex_destroy,			      
+	gcry_gthread_mutex_lock, gcry_gthread_mutex_unlock 
+};
+
+#endif
+
 void 
 _gnome_vfs_ssl_init (void) {
 #ifdef HAVE_OPENSSL
 	SSL_library_init ();
 #elif defined HAVE_GNUTLS
-#error "Need to update the GNUTLS code to support non-blocking fds and cancellation"
+	gcry_control (GCRYCTL_SET_THREAD_CBS, &gcry_threads_gthread);
 	gnutls_global_init();
 #endif
 }
@@ -112,7 +151,29 @@ gnome_vfs_ssl_enabled (void)
 #endif
 }
 
-#ifdef HAVE_OPENSSL
+
+#ifdef HAVE_GNUTLS
+/*
+  From the gnutls documentation:
+
+  To tell you whether a file descriptor should be selected for either reading 
+  or writing, gnutls_record_get_direction() returns 0 if the interrupted 
+  function was trying to read data, and 1 if it was trying to write data.
+
+*/
+
+#ifndef SSL_ERROR_WANT_READ
+#define SSL_ERROR_WANT_READ 0
+#endif
+
+#ifndef SSL_ERROR_WANT_WRITE
+#define SSL_ERROR_WANT_WRITE 1
+#endif
+
+#endif
+
+#if defined(HAVE_OPENSSL) || defined(HAVE_GNUTLS)
+
 static GnomeVFSResult
 handle_ssl_read_write (int fd, int error, struct timeval *timeout,
 		       GnomeVFSCancellation *cancellation)
@@ -150,7 +211,7 @@ handle_ssl_read_write (int fd, int error, struct timeval *timeout,
 	}
 
 	res = select (max_fd + 1, &read_fds, &write_fds, NULL, 
-			timeout ? &tout : NULL);
+		      timeout ? &tout : NULL);
 	
 	if (res == -1 && errno == EINTR) {
 		goto retry;
@@ -167,6 +228,7 @@ handle_ssl_read_write (int fd, int error, struct timeval *timeout,
 	}
 	return GNOME_VFS_ERROR_INTERNAL;
 }
+
 #endif
 
 /**
@@ -234,8 +296,8 @@ gnome_vfs_ssl_create (GnomeVFSSSL **handle_return,
 	if (sock < 0)
 		return gnome_vfs_result_from_errno ();
 
-	_gnome_vfs_set_fd_flags (sock, O_NONBLOCK);
-
+	_gnome_vfs_socket_set_blocking (sock, FALSE);
+	
 	gnome_vfs_address_free (address);
 	
 	return gnome_vfs_ssl_create_from_fd (handle_return, sock, cancellation);
@@ -255,6 +317,7 @@ static const int kx_priority[] =
 	{GNUTLS_KX_DHE_RSA, GNUTLS_KX_RSA, GNUTLS_KX_DHE_DSS, 0};
 static const int mac_priority[] =
 	{GNUTLS_MAC_SHA, GNUTLS_MAC_MD5, 0};
+
 #endif
 
 
@@ -342,6 +405,7 @@ gnome_vfs_ssl_create_from_fd (GnomeVFSSSL **handle_return,
 	return GNOME_VFS_OK;
 
 #elif defined HAVE_GNUTLS
+	GnomeVFSResult res;
 	GnomeVFSSSL *ssl;
 	int err;
 
@@ -371,10 +435,18 @@ gnome_vfs_ssl_create_from_fd (GnomeVFSSSL **handle_return,
 	gnutls_cred_set (ssl->private->tlsstate, GNUTLS_CRD_CERTIFICATE,
 			 ssl->private->xcred);
 
+ retry:
+	res = GNOME_VFS_ERROR_IO;
 	err = gnutls_handshake (ssl->private->tlsstate);
 
-	while (err == GNUTLS_E_AGAIN || err == GNUTLS_E_INTERRUPTED) {
-		err = gnutls_handshake (ssl->private->tlsstate);
+	if (err == GNUTLS_E_INTERRUPTED || err == GNUTLS_E_AGAIN) {
+
+		res = handle_ssl_read_write (ssl->private->sockfd,
+					     gnutls_record_get_direction (ssl->private->tlsstate),
+					     ssl->private->timeout,
+					     cancellation);
+		if (res == GNOME_VFS_OK)
+			goto retry;
 	}
 
 	if (err < 0) {
@@ -382,7 +454,7 @@ gnome_vfs_ssl_create_from_fd (GnomeVFSSSL **handle_return,
 		gnutls_deinit (ssl->private->tlsstate);
 		g_free (ssl->private);
 		g_free (ssl);
-		return GNOME_VFS_ERROR_IO;
+		return res;
 	}
 
 	*handle_return = ssl;
@@ -414,8 +486,9 @@ gnome_vfs_ssl_read (GnomeVFSSSL *ssl,
 		    GnomeVFSCancellation *cancellation)
 {
 #ifdef HAVE_OPENSSL
-	int ret, error;
 	GnomeVFSResult res;
+	int ret;
+	int error;
 	
 	if (bytes == 0) {
 		*bytes_read = 0;
@@ -453,18 +526,41 @@ gnome_vfs_ssl_read (GnomeVFSSSL *ssl,
 	
 	return GNOME_VFS_OK;
 #elif defined HAVE_GNUTLS
+	GnomeVFSResult res;
+	int ret;
+
 	if (bytes == 0) {
 		*bytes_read = 0;
 		return GNOME_VFS_OK;
 	}
 
-	*bytes_read = gnutls_record_recv (ssl->private->tlsstate, buffer, bytes);
+	res = GNOME_VFS_OK;
+retry:
+	ret = gnutls_record_recv (ssl->private->tlsstate, buffer, bytes);
+
+	if (ret == 0) {
+		res = GNOME_VFS_ERROR_EOF;
+	} else if (ret < 0) {
+
+		res = GNOME_VFS_ERROR_IO;
+
+		if (ret == GNUTLS_E_INTERRUPTED || ret == GNUTLS_E_AGAIN) {
+			res = handle_ssl_read_write (ssl->private->sockfd,
+						     gnutls_record_get_direction (ssl->private->tlsstate),
+						     ssl->private->timeout,
+						     cancellation);
+
+			if (res == GNOME_VFS_OK) {
+				goto retry;
+			}
+		}
 
-	if (*bytes_read <= 0) {
 		*bytes_read = 0;
-		return GNOME_VFS_ERROR_GENERIC;
+		return res;
 	}
-	return GNOME_VFS_OK;
+	
+	*bytes_read = ret;
+	return res;
 #else
 	return GNOME_VFS_ERROR_NOT_SUPPORTED;
 #endif
@@ -491,8 +587,9 @@ gnome_vfs_ssl_write (GnomeVFSSSL *ssl,
 		     GnomeVFSCancellation *cancellation)
 {
 #ifdef HAVE_OPENSSL
-	int ret, error;
 	GnomeVFSResult res;
+	int ret;
+	int error;
 	
 	if (bytes == 0) {
 		*bytes_written = 0;
@@ -525,18 +622,41 @@ gnome_vfs_ssl_write (GnomeVFSSSL *ssl,
 	*bytes_written = ret;
 	return GNOME_VFS_OK;
 #elif defined HAVE_GNUTLS
+	GnomeVFSResult res;
+	int ret;
+
 	if (bytes == 0) {
 		*bytes_written = 0;
 		return GNOME_VFS_OK;
 	}
 
-	*bytes_written = gnutls_record_send (ssl->private->tlsstate, buffer, bytes);
+	res = GNOME_VFS_OK;
+retry:
+	ret = gnutls_record_send (ssl->private->tlsstate, buffer, bytes);
+
+	if (ret == 0) {
+		res = GNOME_VFS_ERROR_EOF;
+	} else if (ret < 0) {
+
+		res = GNOME_VFS_ERROR_IO;
+
+		if (ret == GNUTLS_E_INTERRUPTED || ret == GNUTLS_E_AGAIN) {
+			res = handle_ssl_read_write (ssl->private->sockfd,
+						     gnutls_record_get_direction (ssl->private->tlsstate),
+						     ssl->private->timeout,
+						     cancellation);
+
+			if (res == GNOME_VFS_OK) {
+				goto retry;
+			}
+		}
 
-	if (*bytes_written <= 0) {
 		*bytes_written = 0;
-		return GNOME_VFS_ERROR_GENERIC;
+		return res;
 	}
-	return GNOME_VFS_OK;
+	
+	*bytes_written = ret;
+	return res;
 #else
 	return GNOME_VFS_ERROR_NOT_SUPPORTED;
 #endif
@@ -554,8 +674,9 @@ gnome_vfs_ssl_destroy (GnomeVFSSSL *ssl,
 		       GnomeVFSCancellation *cancellation) 
 {
 #ifdef HAVE_OPENSSL
-	int ret, error;
-	GnomeVFSResult res;
+	int ret;
+	GnomeVFSResult res;	
+	int error;
 
  retry:
 	ret = SSL_shutdown (ssl->private->ssl);
@@ -579,7 +700,21 @@ gnome_vfs_ssl_destroy (GnomeVFSSSL *ssl,
 	if (ssl->private->timeout)
 		g_free (ssl->private->timeout);
 #elif defined HAVE_GNUTLS
-	gnutls_bye (ssl->private->tlsstate, GNUTLS_SHUT_RDWR);
+	int ret;
+	GnomeVFSResult res;
+	
+ retry:
+	ret = gnutls_bye (ssl->private->tlsstate, GNUTLS_SHUT_RDWR);
+	
+	if (ret == GNUTLS_E_AGAIN || ret == GNUTLS_E_INTERRUPTED) {
+		res = handle_ssl_read_write (ssl->private->sockfd,
+					     gnutls_record_get_direction (ssl->private->tlsstate),
+					     ssl->private->timeout,
+					     cancellation);
+		if (res == GNOME_VFS_OK)
+			goto retry;
+	}
+
 	gnutls_certificate_free_credentials (ssl->private->xcred);
 	gnutls_deinit (ssl->private->tlsstate);
 	close (ssl->private->sockfd);
@@ -613,7 +748,7 @@ gnome_vfs_ssl_set_timeout (GnomeVFSSSL *ssl,
 			   GTimeVal *timeout,
 			   GnomeVFSCancellation *cancellation)
 {
-#ifdef HAVE_OPENSSL
+#if defined(HAVE_OPENSSL) || defined(HAVE_GNUTLS)
 	/* reset a timeout */
 	if (timeout == NULL) {
 		if (ssl->private->timeout != NULL) {
