@@ -32,15 +32,18 @@
 #endif
 #include "gnome-vfs-utils.h"
 
-#include "gnome-vfs-i18n.h"
 #include "gnome-vfs-private-utils.h"
 #include "gnome-vfs-ops.h"
 #include "gnome-vfs-mime-handlers.h"
 #include "gnome-vfs-mime-private.h"
-#include <glib/gstrfuncs.h>
-#include <glib/gutils.h>
+#include "gnome-vfs-method.h"
+#include <glib.h>
+#include <glib/gi18n-lib.h>
+#include <glib/gstdio.h>
 #include <gconf/gconf-client.h>
+#ifndef G_OS_WIN32
 #include <pwd.h>
+#endif
 #include <stdlib.h>
 #include <string.h>
 #include <sys/param.h>
@@ -48,16 +51,6 @@
 #include <sys/types.h>
 #include <unistd.h>
 
-#if HAVE_SYS_STATVFS_H
-#include <sys/statvfs.h>
-#endif
-
-#if HAVE_SYS_VFS_H
-#include <sys/vfs.h>
-#elif HAVE_SYS_MOUNT_H
-#include <sys/mount.h>
-#endif
-
 #define KILOBYTE_FACTOR 1024.0
 #define MEGABYTE_FACTOR (1024.0 * 1024.0)
 #define GIGABYTE_FACTOR (1024.0 * 1024.0 * 1024.0)
@@ -112,13 +105,19 @@ typedef enum {
 } UnsafeCharacterSet;
 
 static const guchar acceptable[96] =
-{ /* X0   X1   X2   X3   X4   X5   X6   X7   X8   X9   XA   XB   XC   XD   XE   XF */
-    0x00,0x3F,0x20,0x20,0x20,0x00,0x2C,0x3F,0x3F,0x3F,0x3F,0x22,0x20,0x3F,0x3F,0x1C, /* 2X  !"#$%&'()*+,-./   */
-    0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x38,0x20,0x20,0x2C,0x20,0x2C, /* 3X 0123456789:;<=>?   */
-    0x30,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F, /* 4X @ABCDEFGHIJKLMNO   */
-    0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x20,0x20,0x20,0x20,0x3F, /* 5X PQRSTUVWXYZ[\]^_   */
-    0x20,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F, /* 6X `abcdefghijklmno   */
-    0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x20,0x20,0x20,0x3F,0x20  /* 7X pqrstuvwxyz{|}~DEL */
+{ /*  X0   X1   X2   X3   X4   X5   X6   X7   X8   X9   XA   XB   XC   XD   XE   XF */
+  /*   !    "    #    $    %    &    '    (    )    *    +    ,         -    .    / */
+    0x00,0x3F,0x20,0x20,0x20,0x00,0x2C,0x3F,0x3F,0x3F,0x3F,0x22,0x20,0x3F,0x3F,0x1C, 
+  /*   0    1    2    3    4    5    6    7    8    9    :    ;    <    =    >    ? */
+    0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x38,0x20,0x20,0x2C,0x20,0x2C, 
+  /*   @    A    B    C    D    E    F    G    H    I    J    K    L    M    N    O */
+    0x30,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F, 
+  /*   P    Q    R    S    T    U    V    W    X    Y    Z    [    \    ]    ^    _ */
+    0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x20,0x20,0x20,0x20,0x3F, 
+  /*   `    a    b    c    d    e    f    g    h    i    j    k    l    m    n    o */
+    0x20,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F, 
+  /*   p    q    r    s    t    u    v    w    x    y    z    {    |    }    ~  DEL */
+    0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x3F,0x20,0x20,0x20,0x3F,0x20  
 };
 
 enum {
@@ -274,7 +273,34 @@ gnome_vfs_escape_string (const gchar *string)
 gchar *
 gnome_vfs_escape_path_string (const gchar *path)
 {
+#ifdef G_OS_WIN32
+	gchar *path_copy = g_strdup (path);
+	gchar *p = path_copy;
+	gchar *retval;
+
+	/* Replace backslashes with slashes, that is what is used in
+	 * Explorer's "Internet Shortcuts" (.url files), so presumably
+	 * it's the standard.
+	 */
+	while (*p) {
+		if (*p == '\\')
+			*p = '/';
+		p++;
+	}
+	/* Paths starting with a drive letter get an extra leading slash */
+	if (g_ascii_isalpha (path_copy[0]) && path_copy[1] == ':') {
+		gchar *escaped_rest = gnome_vfs_escape_string_internal (path_copy+2, UNSAFE_PATH);
+		path_copy[2] = '\0';
+		retval = g_strconcat ("/", path_copy, escaped_rest, NULL);
+		g_free (escaped_rest);
+	} else
+		retval = gnome_vfs_escape_string_internal (path_copy, UNSAFE_PATH);
+	g_free (path_copy);
+
+	return retval;
+#else
 	return gnome_vfs_escape_string_internal (path, UNSAFE_PATH);
+#endif
 }
 
 /**
@@ -387,6 +413,7 @@ gnome_vfs_escape_set (const char *string,
 char *
 gnome_vfs_expand_initial_tilde (const char *path)
 {
+#ifndef G_OS_WIN32
 	char *slash_after_user_name, *user_name;
 	struct passwd *passwd_file_entry;
 
@@ -417,6 +444,9 @@ gnome_vfs_expand_initial_tilde (const char *path)
 	return g_strconcat (passwd_file_entry->pw_dir,
 			    slash_after_user_name,
 			    NULL);
+#else
+	return g_strdup (path);
+#endif
 }
 
 static int
@@ -451,15 +481,15 @@ unescape_character (const char *scanner)
  * gnome_vfs_unescape_string:
  * @escaped_string: an escaped URI, path, or other string
  * @illegal_characters: a string containing a sequence of characters
- * considered "illegal", '\0' is automatically in this list.
+ * considered "illegal" to be escaped, '\0' is automatically in this list.
  *
  * Decodes escaped characters (i.e. PERCENTxx sequences) in @escaped_string.
  * Characters are encoded in PERCENTxy form, where xy is the ASCII hex code 
  * for character 16x+y.
  * 
- * Return value: a newly allocated string with the unescaped equivalents, 
- * or %NULL if @escaped_string contained one of the characters 
- * in @illegal_characters.
+ * Return value: a newly allocated string with the unescaped
+ * equivalents, or %NULL if @escaped_string contained an escaped
+ * encoding of one of the characters in @illegal_characters.
  **/
 char *
 gnome_vfs_unescape_string (const gchar *escaped_string, 
@@ -600,9 +630,9 @@ gnome_vfs_remove_optional_escapes (char *uri)
 	
 	length = strlen (uri);
 
-	for (scanner = uri; *scanner != '\0'; scanner++, length--) {
+	for (scanner = (guchar *)uri; *scanner != '\0'; scanner++, length--) {
 		if (*scanner == HEX_ESCAPE) {
-			character = unescape_character (scanner + 1);
+			character = unescape_character ((char *)scanner + 1);
 			if (character < 0) {
 				/* invalid hexadecimal character */
 				return GNOME_VFS_ERROR_INVALID_URI;
@@ -739,7 +769,9 @@ gnome_vfs_get_local_path_from_uri (const char *uri)
 
 /**
  * gnome_vfs_get_uri_from_local_path:
- * @local_full_path: a full local filesystem path (i.e. not relative)
+ * @local_full_path: a full local filesystem path (i.e. not relative). 
+ *                   On Windows this should be in the UTF-8 encoding, and
+ *		     can start with a drive letter, but doesn't have to.
  * 
  * Returns a file:/// URI for the local path @local_full_path.
  *
@@ -755,9 +787,10 @@ gnome_vfs_get_uri_from_local_path (const char *local_full_path)
 		return NULL;
 	}
 
-	g_return_val_if_fail (local_full_path[0] == '/', NULL);
+	g_return_val_if_fail (g_path_is_absolute (local_full_path), NULL);
 
 	escaped_path = gnome_vfs_escape_path_string (local_full_path);
+
 	result = g_strconcat ("file://", escaped_path, NULL);
 	g_free (escaped_path);
 	return result;
@@ -765,99 +798,23 @@ gnome_vfs_get_uri_from_local_path (const char *local_full_path)
 
 /**
  * gnome_vfs_get_volume_free_space:
- * @vfs_uri:
- * @size:
+ * @vfs_uri: 
+ * @size: 
  * 
- * Stores in @size the amount of free space on a volume.
- * This only works for URIs with the file: scheme.
+ * Stores the amount of free space in bytes on @vfs-uri's
+ * volume in @size.
  *
- * Returns: GNOME_VFS_OK on success, otherwise an error code
+ * Returns: GNOME_VFS_OK on success, otherwise an
+ *          GNOME_VFS_ERROR_* code
  */
 GnomeVFSResult
 gnome_vfs_get_volume_free_space (const GnomeVFSURI *vfs_uri, 
 				 GnomeVFSFileSize  *size)
 {	
-	GnomeVFSFileSize free_blocks, block_size;
-       	int statfs_result;
-	const char *path, *scheme;
-	char *unescaped_path;
-
-#if HAVE_STATVFS
-	struct statvfs statfs_buffer;
-#else
-	struct statfs statfs_buffer;
-#endif
-
- 	*size = 0;
-
-	path = gnome_vfs_uri_get_path (vfs_uri);
-	if (path == NULL) {
+	if (!VFS_METHOD_HAS_FUNC(vfs_uri->method, get_volume_free_space))
 		return GNOME_VFS_ERROR_NOT_SUPPORTED;
-	}
 
-	scheme = gnome_vfs_uri_get_scheme (vfs_uri);
-	
-        /* We only handle the file scheme for now */
-	if (g_ascii_strcasecmp (scheme, "file") != 0 || !_gnome_vfs_istr_has_prefix (path, "/")) {
-		return GNOME_VFS_ERROR_NOT_SUPPORTED;
-	}
-
-	unescaped_path = gnome_vfs_unescape_string (path, G_DIR_SEPARATOR_S);
-	
-#if HAVE_STATVFS
-	statfs_result = statvfs (unescaped_path, &statfs_buffer);
-	block_size = statfs_buffer.f_frsize; 
-#else
-#if STATFS_ARGS == 2
-	statfs_result = statfs (unescaped_path, &statfs_buffer);
-#elif STATFS_ARGS == 4
-	statfs_result = statfs (unescaped_path, &statfs_buffer,
-				sizeof (statfs_buffer), 0);
-#endif
-	block_size = statfs_buffer.f_bsize; 
-#endif  
-
-	if (statfs_result != 0) {
-		g_free (unescaped_path);
-		return gnome_vfs_result_from_errno ();
-	}
-
-
-/* CF: I assume ncpfs is linux specific, if you are on a non-linux platform
- * where ncpfs is available, please file a bug about it on bugzilla.gnome.org
- * (2004-03-08)
- */
-#if defined(__linux__)
-	/* ncpfs does not know the amount of available and free space */
-	if (statfs_buffer.f_bavail == 0 && statfs_buffer.f_bfree == 0) {
-		/* statvfs does not contain an f_type field, we try again
-		 * with statfs.
-		 */
-		struct statfs statfs_buffer2;
-		statfs_result = statfs (unescaped_path, &statfs_buffer2);
-		g_free (unescaped_path);
-
-		if (statfs_result != 0) {
-			return gnome_vfs_result_from_errno ();
-		}
-		
-		/* linux/ncp_fs.h: NCP_SUPER_MAGIC == 0x564c */
-		if (statfs_buffer2.f_type == 0x564c)
-		{
-			return GNOME_VFS_ERROR_NOT_SUPPORTED;
-		}
-	} else {
-		/* everything is copacetic... free the unescaped path */
-		g_free (unescaped_path);
-	}
-#else 	
-	g_free (unescaped_path);
-#endif
-	free_blocks = statfs_buffer.f_bavail;
-
-	*size = block_size * free_blocks;
-	
-	return GNOME_VFS_OK;
+	return vfs_uri->method->get_volume_free_space (vfs_uri->method, vfs_uri, size);
 }
 
 char *
@@ -874,13 +831,17 @@ gnome_vfs_icon_path_from_filename (const char *relative_filename)
 	gnome_var = g_getenv ("GNOME_PATH");
 
 	if (gnome_var == NULL) {
-		gnome_var = PREFIX;
+		gnome_var = GNOME_VFS_PREFIX;
 	}
 
-	paths = g_strsplit (gnome_var, ":", 0); 
+	paths = g_strsplit (gnome_var, G_SEARCHPATH_SEPARATOR_S, 0); 
 
 	for (temp_paths = paths; *temp_paths != NULL; temp_paths++) {
-		full_filename = g_strconcat (*temp_paths, "/share/pixmaps/", relative_filename, NULL);
+		full_filename = g_build_filename (*temp_paths,
+						  "share",
+						  "pixmaps",
+						  relative_filename,
+						  NULL);
 		if (g_file_test (full_filename, G_FILE_TEST_EXISTS)) {
 			g_strfreev (paths);
 			return full_filename;
@@ -905,20 +866,13 @@ strdup_to (const char *string, const char *end)
 static gboolean
 is_executable_file (const char *path)
 {
-	struct stat stat_buffer;
-
-	/* Check that it exists. */
-	if (stat (path, &stat_buffer) != 0) {
-		return FALSE;
-	}
-
-	/* Check that it is a file. */
-	if (!S_ISREG (stat_buffer.st_mode)) {
+	/* Check that it exists and is regular. */
+	if (!g_file_test (path, G_FILE_TEST_IS_REGULAR)) {
 		return FALSE;
 	}
 
 	/* Check that it's executable. */
-	if (access (path, X_OK) != 0) {
+	if (!g_file_test (path, G_FILE_TEST_IS_EXECUTABLE)) {
 		return FALSE;
 	}
 
@@ -936,7 +890,7 @@ executable_in_path (const char *executable_name)
 
 	for (piece_start = path_list; ; piece_start = piece_end + 1) {
 		/* Find the next piece of PATH. */
-		piece_end = strchr (piece_start, ':');
+		piece_end = strchr (piece_start, G_SEARCHPATH_SEPARATOR);
 		piece = strdup_to (piece_start, piece_end);
 		g_strstrip (piece);
 		
@@ -944,7 +898,12 @@ executable_in_path (const char *executable_name)
 			is_good = FALSE;
 		} else {
 			/* Try out this path with the executable. */
-			raw_path = g_strconcat (piece, "/", executable_name, NULL);
+			raw_path = g_build_filename (piece, executable_name, NULL);
+			/* XXX Can this be necessary? Surely PATH
+			 * isn't supposed to contain actual tildes,
+			 * but the shell expands any tilde one tries
+			 * to add to PATH?
+			 */ 
 			expanded_path = gnome_vfs_expand_initial_tilde (raw_path);
 			g_free (raw_path);
 			
@@ -990,7 +949,7 @@ gnome_vfs_is_executable_command_string (const char *command_string)
 	gboolean found;
 
 	/* Check whether command_string is a full path for an executable. */
-	if (command_string[0] == '/') {
+	if (g_path_is_absolute (command_string)) {
 
 		/* FIXME bugzilla.eazel.com 2757:
 		 * Because we don't handle quoting, we can check for full
@@ -1030,11 +989,13 @@ gnome_vfs_is_executable_command_string (const char *command_string)
  * Reads an entire file into memory for convenience. Beware accidentally
  * loading large files into memory with this function.
  *
+ * Since version 2.10 the string stored in @file_contents will be nul-terminated,
+ * so for text files you can use result as a normal string.
+ *
  * Return value: An integer representing the result of the operation
  *
  * Since: 2.2
  */
-
 GnomeVFSResult
 gnome_vfs_read_entire_file (const char *uri,
 			    int *file_size,
@@ -1046,10 +1007,10 @@ gnome_vfs_read_entire_file (const char *uri,
 	GnomeVFSFileSize total_bytes_read;
 	GnomeVFSFileSize bytes_read;
 
-	g_assert (file_size != NULL);
 	g_assert (file_contents != NULL);
 
-	*file_size = 0;
+	if (file_size != NULL)
+		*file_size = 0;
 	*file_contents = NULL;
 
 	/* Open the file. */
@@ -1083,6 +1044,9 @@ gnome_vfs_read_entire_file (const char *uri,
 		total_bytes_read += bytes_read;
 	} while (result == GNOME_VFS_OK);
 
+	buffer = g_realloc (buffer, total_bytes_read + 1);
+	buffer[total_bytes_read] = '\0';
+
 	/* Close the file. */
 	result = gnome_vfs_close (handle);
 	if (result != GNOME_VFS_OK) {
@@ -1091,8 +1055,9 @@ gnome_vfs_read_entire_file (const char *uri,
 	}
 
 	/* Return the file. */
-	*file_size = total_bytes_read;
-	*file_contents = g_realloc (buffer, total_bytes_read);
+	if (file_size != NULL)
+		*file_size = total_bytes_read;
+	*file_contents = g_realloc (buffer, total_bytes_read + 1);
 	return GNOME_VFS_OK;
 }
 
@@ -1330,14 +1295,14 @@ gnome_vfs_escape_high_chars (const guchar *string)
 	}
 	
 	if (escape_count == 0) {
-		return g_strdup (string);
+		return g_strdup ((char *)string);
 	}
 
 	/* allocate two extra characters for every character that
 	 * needs escaping and space for a trailing zero
 	 */
 	result = g_malloc (scanner - string + escape_count * 2 + 1);
-	for (scanner = string, result_scanner = result; *scanner != '\0'; scanner++) {
+	for (scanner = string, result_scanner = (guchar *)result; *scanner != '\0'; scanner++) {
 		if (!ACCEPTABLE(*scanner)) {
 			*result_scanner++ = '%';
 			*result_scanner++ = hex[*scanner >> 4];
@@ -1390,7 +1355,7 @@ gnome_vfs_make_uri_from_input_internal (const char *text,
 					const char *filename_charset,
 					gboolean strip_trailing_whitespace)
 {
-	char *stripped, *path, *uri, *locale_path, *filesystem_path, *escaped;
+	char *stripped, *uri, *locale_path, *escaped;
 
 	g_return_val_if_fail (text != NULL, g_strdup (""));
 
@@ -1404,11 +1369,7 @@ gnome_vfs_make_uri_from_input_internal (const char *text,
 		stripped = g_strchug (g_strdup (text));
 	}
 
-	switch (stripped[0]) {
-	case '\0':
-		uri = g_strdup ("");
-		break;
-	case '/':
+	if (g_path_is_absolute (stripped)) {
 		if (!filenames_are_utf8) {
 			locale_path = g_convert (stripped, -1, filename_charset, "UTF-8", NULL, NULL, NULL);
 			if (locale_path != NULL) {
@@ -1422,8 +1383,13 @@ gnome_vfs_make_uri_from_input_internal (const char *text,
 		} else {
 			uri = gnome_vfs_get_uri_from_local_path (stripped);
 		}
+	} else switch (stripped[0]) {
+	case '\0':
+		uri = g_strdup ("");
 		break;
-	case '~':
+#ifndef G_OS_WIN32
+	case '~': {
+		char *path, *filesystem_path;
 		if (!filenames_are_utf8) {
 			filesystem_path = g_convert (stripped, -1, filename_charset, "UTF-8", NULL, NULL, NULL);
 		} else {
@@ -1441,15 +1407,17 @@ gnome_vfs_make_uri_from_input_internal (const char *text,
 			g_free (path);
 		}
                 /* don't insert break here, read above comment */
+	}
+#endif
 	default:
 		if (has_valid_scheme (stripped)) {
-			uri = gnome_vfs_escape_high_chars (stripped);
+			uri = gnome_vfs_escape_high_chars ((guchar *)stripped);
 		} else if (looks_like_http_uri (stripped)) {
-			escaped = gnome_vfs_escape_high_chars (stripped);
+			escaped = gnome_vfs_escape_high_chars ((guchar *)stripped);
 			uri = g_strconcat ("http://", escaped, NULL);
 			g_free (escaped);
 		} else {
-			escaped = gnome_vfs_escape_high_chars (stripped);
+			escaped = gnome_vfs_escape_high_chars ((guchar *)stripped);
 			uri = g_strconcat ("file://", escaped, NULL);
 			g_free (escaped);
 		}
@@ -1463,7 +1431,7 @@ gnome_vfs_make_uri_from_input_internal (const char *text,
 
 /**
  * gnome_vfs_make_uri_from_input:
- * @uri: a possibly mangled "uri", in UTF8
+ * @location: a possibly mangled "uri", in UTF8
  *
  * Takes a user input path/URI and makes a valid URI out of it.
  *
@@ -1486,10 +1454,34 @@ gnome_vfs_make_uri_from_input (const char *location)
 	return gnome_vfs_make_uri_from_input_internal (location, utf8, charset, TRUE);
 }
 
+/**
+ * gnome_vfs_make_uri_from_input_with_trailing_ws:
+ * @location: a possibly mangled "uri", in UTF8
+ *
+ * Takes a user input path/URI and makes a valid URI out of it.
+ *
+ * This function is indentical to #gnome_vfs_make_uri_from_input except
+ * that this version won't strip any trailing slashes.
+ *
+ * Returns: a newly allocated uri.
+ *
+ * Since: 2.12
+ **/
+char *
+gnome_vfs_make_uri_from_input_with_trailing_ws (const char *location)
+{
+	gboolean utf8;
+	const char *charset;
+
+	utf8 = vfs_get_filename_charset (&charset);
+
+	return gnome_vfs_make_uri_from_input_internal (location, utf8, charset, FALSE);	
+}
+
 /**
  * gnome_vfs_make_uri_from_input_with_dirs:
- * @uri: a relative or absolute path
- * @dirs: directory to use as a base directory if @uri is a relative path.
+ * @location: a relative or absolute path
+ * @dirs: directory to use as a base directory if @location is a relative path.
  *
  * Determines a fully qualified URL from a relative or absolute input path.
  * Basically calls gnome_vfs_make_uri_from_input except it specifically
@@ -1501,21 +1493,22 @@ gnome_vfs_make_uri_from_input (const char *location)
  * Since: 2.4
  */
 char *
-gnome_vfs_make_uri_from_input_with_dirs (const char *in,
+gnome_vfs_make_uri_from_input_with_dirs (const char *location,
 					 GnomeVFSMakeURIDirs dirs)
 {
 	char *uri, *path, *dir;
 
-	switch (in[0]) {
+	if (g_path_is_absolute (location))
+		uri = gnome_vfs_make_uri_from_input (location);
+	else switch (location[0]) {
 	case '\0':
 		uri = g_strdup ("");
 		break;
-		
+#ifndef G_OS_WIN32		
 	case '~':
-	case '/':
-		uri = gnome_vfs_make_uri_from_input (in);
+		uri = gnome_vfs_make_uri_from_input (location);
 		break;
-		
+#endif
 	default:
 		/* this might be a relative path, check if it exists relative
 		 * to current dir and home dir.
@@ -1523,7 +1516,7 @@ gnome_vfs_make_uri_from_input_with_dirs (const char *in,
 		uri = NULL;
 		if (dirs & GNOME_VFS_MAKE_URI_DIR_CURRENT) {
 			dir = g_get_current_dir ();
-			path = g_build_filename (dir, in, NULL);
+			path = g_build_filename (dir, location, NULL);
 			g_free (dir);
 			
 			if (g_file_test (path, G_FILE_TEST_EXISTS)) {
@@ -1534,7 +1527,7 @@ gnome_vfs_make_uri_from_input_with_dirs (const char *in,
 
 		if (uri == NULL &&
 		    dirs & GNOME_VFS_MAKE_URI_DIR_HOMEDIR) {
-			path = g_build_filename (g_get_home_dir (), in, NULL);
+			path = g_build_filename (g_get_home_dir (), location, NULL);
 		
 			if (g_file_test (path, G_FILE_TEST_EXISTS)) {
 				uri = gnome_vfs_make_uri_from_input (path);
@@ -1543,7 +1536,7 @@ gnome_vfs_make_uri_from_input_with_dirs (const char *in,
 		}
 
 		if (uri == NULL) {
-			uri = gnome_vfs_make_uri_from_input (in);
+			uri = gnome_vfs_make_uri_from_input (location);
 		}
 	}
 	
@@ -1769,11 +1762,19 @@ gnome_vfs_make_uri_canonical (const char *uri)
 	 * the rules are more strict.
 	 */
 
+#ifndef G_OS_WIN32
+#define URI_CONTAINS_NO_SCHEME(uri) (strchr (uri, ':') == NULL)
+#else
+#define URI_CONTAINS_NO_SCHEME(uri) \
+  (strlen (uri) > 2 && strchr (uri + 2, ':') == NULL)
+#endif
+
+
 	/* Add file: if there is no scheme. */
-	if (strchr (canonical_uri, ':') == NULL) {
+	if (URI_CONTAINS_NO_SCHEME (canonical_uri)) {
 		old_uri = canonical_uri;
 
-		if (old_uri[0] != '/') {
+		if (!g_path_is_absolute (old_uri)) {
 			/* FIXME bugzilla.eazel.com 5069: 
 			 *  bandaid alert. Is this really the right thing to do?
 			 * 
@@ -1904,13 +1905,12 @@ gnome_vfs_make_uri_from_shell_arg (const char *location)
 
 	g_return_val_if_fail (location != NULL, g_strdup (""));
 
-	switch (location[0]) {
+	if (g_path_is_absolute (location))
+		uri = gnome_vfs_get_uri_from_local_path (location);
+	else switch (location[0]) {
 	case '\0':
 		uri = g_strdup ("");
 		break;
-	case '/':
-		uri = gnome_vfs_get_uri_from_local_path (location);
-		break;
 	default:
 		if (has_valid_scheme (location)) {
 			uri = g_strdup (location);
