@@ -90,6 +90,16 @@ gnome_vfs_volume_init (GnomeVFSVolume *volume)
 	
 }
 
+/** 
+ * gnome_vfs_volume_ref:
+ * @volume:
+ *
+ *
+ *
+ * Returns:
+ *
+ * Since: 2.6
+ */
 GnomeVFSVolume *
 gnome_vfs_volume_ref (GnomeVFSVolume *volume)
 {
@@ -103,6 +113,14 @@ gnome_vfs_volume_ref (GnomeVFSVolume *volume)
 	return volume;
 }
 
+/** 
+ * gnome_vfs_volume_unref:
+ * @volume:
+ *
+ *
+ *
+ * Since: 2.6
+ */
 void
 gnome_vfs_volume_unref (GnomeVFSVolume *volume)
 {
@@ -136,6 +154,8 @@ gnome_vfs_volume_finalize (GObject *object)
 	g_free (priv->display_name);
 	g_free (priv->icon);
 	g_free (priv->gconf_id);
+	g_free (priv->hal_udi);
+	g_free (priv->hal_drive_udi);
 	g_free (priv);
 	volume->priv = NULL;
 	
@@ -143,24 +163,66 @@ gnome_vfs_volume_finalize (GObject *object)
 		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
+
+/** 
+ * gnome_vfs_volume_get_volume_type:
+ * @volume:
+ *
+ *
+ *
+ * Returns:
+ *
+ * Since: 2.6
+ */
 GnomeVFSVolumeType
 gnome_vfs_volume_get_volume_type (GnomeVFSVolume *volume)
 {
 	return volume->priv->volume_type;
 }
 
+
+/** 
+ * gnome_vfs_volume_get_device_type:
+ * @volume:
+ *
+ *
+ *
+ * Returns:
+ *
+ * Since: 2.6
+ */
 GnomeVFSDeviceType
 gnome_vfs_volume_get_device_type (GnomeVFSVolume *volume)
 {
 	return volume->priv->device_type;
 }
 
+/** 
+ * gnome_vfs_volume_get_id:
+ * @volume:
+ *
+ *
+ *
+ * Returns:
+ *
+ * Since: 2.6
+ */
 gulong
 gnome_vfs_volume_get_id (GnomeVFSVolume *volume)
 {
 	return volume->priv->id;
 }
 
+/** 
+ * gnome_vfs_volume_get_drive:
+ * @volume:
+ *
+ *
+ *
+ * Returns:
+ *
+ * Since: 2.6
+ */
 GnomeVFSDrive *
 gnome_vfs_volume_get_drive (GnomeVFSVolume *volume)
 {
@@ -193,54 +255,160 @@ _gnome_vfs_volume_set_drive           (GnomeVFSVolume     *volume,
 	G_UNLOCK (volumes);
 }
 
+/** 
+ * gnome_vfs_volume_get_device_path:
+ * @volume:
+ *
+ *
+ *
+ * Returns: a newly allocated string
+ *
+ * Since: 2.6
+ */
 char *
 gnome_vfs_volume_get_device_path (GnomeVFSVolume *volume)
 {
 	return g_strdup (volume->priv->device_path);
 }
 
+/** 
+ * gnome_vfs_volume_get_activation_uri:
+ * @volume:
+ *
+ *
+ *
+ * Returns: a newly allocated string
+ *
+ * Since: 2.6
+ */
 char *
 gnome_vfs_volume_get_activation_uri (GnomeVFSVolume *volume)
 {
 	return g_strdup (volume->priv->activation_uri);
 }
 
+/** 
+ * gnome_vfs_volume_get_hal_udi:
+ * @volume:
+ *
+ *
+ *
+ * Returns: a newly allocated string
+ *
+ * Since: 2.6
+ */
+char *
+gnome_vfs_volume_get_hal_udi (GnomeVFSVolume *volume)
+{
+	return g_strdup (volume->priv->hal_udi);
+}
+
+/** 
+ * gnome_vfs_volume_get_filesystem_type:
+ * @volume:
+ *
+ *
+ *
+ * Returns: a newly allocated string
+ *
+ * Since: 2.6
+ */
 char *
 gnome_vfs_volume_get_filesystem_type (GnomeVFSVolume *volume)
 {
 	return g_strdup (volume->priv->filesystem_type);
 }
 
+/** 
+ * gnome_vfs_volume_get_display_name:
+ * @volume:
+ *
+ *
+ *
+ * Returns: a newly allocated string
+ *
+ * Since: 2.6
+ */
 char *
 gnome_vfs_volume_get_display_name (GnomeVFSVolume *volume)
 {
 	return g_strdup (volume->priv->display_name);
 }
 
+/** 
+ * gnome_vfs_volume_get_icon:
+ * @volume:
+ *
+ *
+ *
+ * Returns: a newly allocated string
+ *
+ * Since: 2.6
+ */
 char *
 gnome_vfs_volume_get_icon (GnomeVFSVolume *volume)
 {
 	return g_strdup (volume->priv->icon);
 }
 
+/** 
+ * gnome_vfs_volume_is_user_visible:
+ * @volume:
+ *
+ *
+ *
+ * Returns:
+ *
+ * Since: 2.6
+ */
 gboolean
 gnome_vfs_volume_is_user_visible (GnomeVFSVolume *volume)
 {
 	return volume->priv->is_user_visible;
 }
 
+/** 
+ * gnome_vfs_volume_is_read_only:
+ * @volume:
+ *
+ *
+ *
+ * Returns:
+ *
+ * Since: 2.6
+ */
 gboolean
 gnome_vfs_volume_is_read_only (GnomeVFSVolume *volume)
 {
 	return volume->priv->is_read_only;
 }
 
+/** 
+ * gnome_vfs_volume_is_mounted:
+ * @volume:
+ *
+ *
+ *
+ * Returns:
+ *
+ * Since: 2.6
+ */
 gboolean
 gnome_vfs_volume_is_mounted (GnomeVFSVolume *volume)
 {
 	return volume->priv->is_mounted;
 }
 
+/** 
+ * gnome_vfs_volume_handles_trash:
+ * @volume:
+ *
+ *
+ *
+ * Returns:
+ *
+ * Since: 2.6
+ */
 gboolean
 gnome_vfs_volume_handles_trash (GnomeVFSVolume *volume)
 {
@@ -285,6 +453,17 @@ _gnome_vfs_device_type_get_sort_group (GnomeVFSDeviceType type)
 	}
 }
 
+/** 
+ * gnome_vfs_volume_compare:
+ * @a:
+ * @b:
+ *
+ *
+ *
+ * Returns:
+ *
+ * Since: 2.6
+ */
 gint
 gnome_vfs_volume_compare (GnomeVFSVolume *a,
 			  GnomeVFSVolume *b)
@@ -334,6 +513,14 @@ decode_corba_string_or_null (CORBA_char *str, gboolean empty_is_null)
 	}
 }
 
+/** 
+ * gnome_vfs_volume_to_corba:
+ * @volume:
+ *
+ *
+ *
+ * Since: 2.6
+ */
 void
 gnome_vfs_volume_to_corba (GnomeVFSVolume *volume,
 			   GNOME_VFS_Volume *corba_volume)
@@ -357,6 +544,7 @@ gnome_vfs_volume_to_corba (GnomeVFSVolume *volume,
 	corba_volume->display_name = corba_string_or_null_dup (volume->priv->display_name);
 	corba_volume->icon = corba_string_or_null_dup (volume->priv->icon);
 	corba_volume->gconf_id = corba_string_or_null_dup (volume->priv->gconf_id);
+	corba_volume->hal_udi = corba_string_or_null_dup (volume->priv->hal_udi);
 	
 	corba_volume->is_user_visible = volume->priv->is_user_visible;
 	corba_volume->is_read_only = volume->priv->is_read_only;
@@ -379,7 +567,7 @@ _gnome_vfs_volume_from_corba (const GNOME_VFS_Volume *corba_volume,
 		volume->priv->drive = gnome_vfs_volume_monitor_get_drive_by_id (volume_monitor,
 										corba_volume->drive);
 		if (volume->priv->drive != NULL) {
-			_gnome_vfs_drive_set_mounted_volume (volume->priv->drive, volume);
+			_gnome_vfs_drive_add_mounted_volume (volume->priv->drive, volume);
 			/* The drive reference is weak */
 			gnome_vfs_drive_unref (volume->priv->drive);
 		}
@@ -392,6 +580,7 @@ _gnome_vfs_volume_from_corba (const GNOME_VFS_Volume *corba_volume,
 	volume->priv->display_name = decode_corba_string_or_null (corba_volume->display_name, TRUE);
 	volume->priv->icon = decode_corba_string_or_null (corba_volume->icon, TRUE);
 	volume->priv->gconf_id = decode_corba_string_or_null (corba_volume->gconf_id, TRUE);
+	volume->priv->hal_udi = decode_corba_string_or_null (corba_volume->hal_udi, TRUE);
 	
 	volume->priv->is_user_visible = corba_volume->is_user_visible;
 	volume->priv->is_read_only = corba_volume->is_read_only;
