@@ -30,18 +30,22 @@
 #include <sys/time.h>
 
 #ifdef HAVE_HOWL
+/* Need to work around howl exporting its config file... */
+#undef PACKAGE
+#undef VERSION
 #include <howl.h>
 #endif
 
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-directory.h>
-#include <libgnomevfs/gnome-vfs-i18n.h>
 #include <libgnomevfs/gnome-vfs-module-shared.h>
 #include <libgnomevfs/gnome-vfs-module.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-dns-sd.h>
 #include <libgnomevfs/gnome-vfs-monitor-private.h>
 
+#include <glib/gi18n-lib.h>
+
 #define BROWSE_TIMEOUT_MSEC 5000
 #define RESOLVE_TIMEOUT_MSEC 5000
 #define LOCAL_SYNC_BROWSE_DELAY_MSEC 200
@@ -54,6 +58,7 @@ static struct {
 } dns_sd_types[] = {
 	{"_ftp._tcp", "ftp", "gnome-fs-ftp"},
 	{"_webdav._tcp", "dav", "gnome-fs-share"},
+	{"_sftp-ssh._tcp", "sftp", "gnome-fs-ssh"},
 };
 
 #ifdef HAVE_HOWL
@@ -287,14 +292,14 @@ local_browse (gboolean add,
 }
 
 static sw_result
-local_browse_callback_sync (sw_discovery_browse_handler  handler,
-			    sw_discovery                 discovery,
-			    sw_discovery_browse_id       id,
+local_browse_callback_sync (sw_discovery                 discovery,
+			    sw_discovery_oid             id,
 			    sw_discovery_browse_status   status,
-			    sw_const_string               name,
-			    sw_const_string               type,
-			    sw_const_string               domain,
-			    sw_opaque                     extra)
+			    sw_uint32			 interface_index,
+			    sw_const_string              name,
+			    sw_const_string              type,
+			    sw_const_string              domain,
+			    sw_opaque                    extra)
 {
 	if (status == SW_DISCOVERY_BROWSE_ADD_SERVICE)
 		local_browse (TRUE, name, type, domain);
@@ -329,7 +334,7 @@ init_local (void)
 		sw_salt salt;
 		sw_ulong timeout;
 		struct timeval end_tv, tv;
-		sw_discovery_browse_id *sync_handles;
+		sw_discovery_oid *sync_handles;
 		int timeout_msec;
 		sw_result swres;
 		
@@ -358,12 +363,12 @@ init_local (void)
 			return;
 		}
 
-		sync_handles = g_new0 (sw_discovery_browse_id, G_N_ELEMENTS (dns_sd_types));
+		sync_handles = g_new0 (sw_discovery_oid, G_N_ELEMENTS (dns_sd_types));
 
 		for (i = 0; i < G_N_ELEMENTS (dns_sd_types); i++) {
 			sw_discovery_browse (session,
+					     0, 
 					     dns_sd_types[i].type, "local",
-					     NULL,
 					     local_browse_callback_sync,
 					     NULL,
 					     &(sync_handles[i]));
@@ -390,7 +395,7 @@ init_local (void)
 		
 		for (i = 0; i < G_N_ELEMENTS (dns_sd_types); i++) {
 			if (sync_handles[i] != 0) {
-				sw_discovery_stop_browse (session, sync_handles[i]);
+				sw_discovery_cancel (session, sync_handles[i]);
 			}
 		}
 					  
@@ -627,7 +632,7 @@ do_seek (GnomeVFSMethod *method,
 static GnomeVFSResult
 do_tell (GnomeVFSMethod *method,
 	 GnomeVFSMethodHandle *method_handle,
-	 GnomeVFSFileOffset *offset_return)
+	 GnomeVFSFileSize *offset_return)
 {
 	FileHandle *file_handle;
 
