@@ -38,6 +38,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
+#include <gconf/gconf-client.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 
@@ -95,10 +96,12 @@ static GHashTable *user_cache = NULL;
 #define CACHE_REAP_TIMEOUT (30 * 60)
 static guint cache_reap_timeout = 0;
 
+/* We load a default workgroup from gconf */
+#define PATH_GCONF_GNOME_VFS_SMB_WORKGROUP "/system/smb/workgroup"
+
 /* The magic "default workgroup" hostname */
 #define DEFAULT_WORKGROUP_NAME "X-GNOME-DEFAULT-WORKGROUP"
 
-
 /* 5 minutes before we re-read the workgroup cache again */
 #define WORKGROUP_CACHE_TIMEOUT (5*60)
 
@@ -110,7 +113,6 @@ static time_t workgroups_timestamp = 0;
 #define SMB_AUTH_STATE_PREFILLED	0x00000010 	/* Have asked gnome-auth for prefilled auth */
 #define SMB_AUTH_STATE_PROMPTED		0x00000020 	/* Have asked gnome-auth for to prompt user */
 
-/* TODO: Move this to the top of the file */
 typedef struct _SmbAuthContext {
 	
 	GnomeVFSURI *uri;		/* Uri being worked with. Does not own this URI */
@@ -604,6 +606,8 @@ static gboolean
 try_init (void)
 {
 	char *path;
+	GConfClient *gclient;
+	gchar *workgroup;
 	struct stat statbuf;
 
 	LOCK_SMB();
@@ -632,6 +636,20 @@ try_init (void)
 		smb_context->callbacks.remove_cached_srv_fn = remove_cached_server;
 		smb_context->callbacks.purge_cached_fn      = purge_cached;
 
+		gclient = gconf_client_get_default ();
+		if (gclient) {
+			workgroup = gconf_client_get_string (gclient, 
+					PATH_GCONF_GNOME_VFS_SMB_WORKGROUP, NULL);
+
+			/* libsmbclient frees this on it's own, so make sure 
+			 * to use simple system malloc */
+			if (workgroup && workgroup[0])
+				smb_context->workgroup = strdup (workgroup);
+			
+			g_free (workgroup);
+			g_object_unref (gclient);
+		}
+
 		if (!smbc_init_context (smb_context)) {
 			smbc_free_context (smb_context, FALSE);
 			smb_context = NULL;
@@ -930,7 +948,8 @@ prompt_authentication (SmbAuthContext *actx)
 	in_args.domain = (char*)actx->use_domain;
 	in_args.port = actx->uri ? ((GnomeVFSToplevelURI*)actx->uri)->host_port : 0;
 
-	/* TODO: set default_user & default_domain? */
+	in_args.default_user = actx->use_user ? actx->use_user : (char*)g_get_user_name ();
+	in_args.default_domain = actx->use_domain ? actx->use_domain : smb_context->workgroup;
 	
 	memset (&out_args, 0, sizeof (out_args));
 
